﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace Sneaky
{
    public partial class Settings : Form
    {
        private readonly global::Sneaky.Sneaky _sneaky;
        private readonly SLogger _sLogger;

        public Settings(global::Sneaky.Sneaky sneakyInstance)
        {
            InitializeComponent();

            _sneaky = sneakyInstance;
            _sLogger = sneakyInstance.Logger;
        }

        internal void ReloadAvoidList()
        {
            // This reloads the units if the form is reopened.
            if(_sneaky.AvoidUnits.Any())
            {
                lbox_avoidlist.Items.Clear();
                foreach(var unit in _sneaky.AvoidUnits)
                {
                    lbox_avoidlist.Items.Add(unit);
                }
            }
        }

        private void ReloadStealthTrigger()
        {
            numericUpDown1.Value = _sneaky.StealthTrigger;
        }

        private void CraftWindow_Load(object sender, EventArgs e)
        {
            ReloadStealthTrigger();
            ReloadAvoidList();
            DisplayNearUnits();

            // Recheck checkboxes on window reload
            if(_sneaky.sForceStealthBeforeHarvest)
            {
                cbox_forceStealthBeforeHarvest.Checked = true;
            }

            if(_sneaky.sDetectNewEnemies)
            {
                cbox_detectNewEnemies.Checked = true;
            }
        }

        /// <summary>
        /// Fills the nearby unit listbox with nearby unit names
        /// </summary>
        internal void DisplayNearUnits()
        {
            _sneaky.NearbyUnits = _sneaky.GetNearUnits();
            if (!_sneaky.NearbyUnits.Any()) return;

            lbox_nearUnitsList.Items.Clear();
            foreach (var unit in _sneaky.NearbyUnits)
            {
                if(lbox_avoidlist.Items.Contains(unit.EnglishName))
                    continue;

                if(!lbox_nearUnitsList.Items.Contains(unit.EnglishName))
                    lbox_nearUnitsList.Items.Add(unit.EnglishName);
            }

        }


        private void btn_refreshNearUnits_Click(object sender, EventArgs e)
        {
            _sneaky.NearbyUnits = _sneaky.GetNearUnits();
            DisplayNearUnits();
        }

        private void btn_addToAvoid_Click(object sender, EventArgs e)
        {
            if (lbox_nearUnitsList.SelectedItems.Count > 0)
            {
                // Move the selected nearby units to the avoid list
                foreach (var nearUnit in lbox_nearUnitsList.SelectedItems)
                {
                    lbox_avoidlist.Items.Add(nearUnit);
                    _sneaky.AvoidUnits.Add(nearUnit.ToString());
                }

                // Remove the moved entries from the nearby unit listbox
                foreach (var avoidUnit in lbox_avoidlist.Items)
                {
                    if (lbox_nearUnitsList.Items.Contains(avoidUnit))
                    {
                        lbox_nearUnitsList.Items.Remove(avoidUnit);
                    }
                }
            }
        }

        private void btn_removeFromAvoid_Click(object sender, EventArgs e)
        {
            if (lbox_avoidlist.SelectedItems.Count <= 0) return;

            foreach (var avoidUnit in lbox_avoidlist.SelectedItems)
            {
                if (_sneaky.AvoidUnits.Contains(avoidUnit))
                {
                    _sneaky.AvoidUnits.Remove(avoidUnit.ToString());
                }
            }

            lbox_avoidlist.Items.Clear();
            foreach(var avoidUnit in _sneaky.AvoidUnits)
            {
                lbox_avoidlist.Items.Add(avoidUnit);
            }

            DisplayNearUnits();
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            _sneaky.StealthTrigger = (int)numericUpDown1.Value;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if(lbox_avoidlist.Items.Count > 0)
            {
                lbox_avoidlist.Items.Clear();
            }
            _sneaky.AvoidUnits.Clear();
        }

        private void cbox_detectNewEnemies_CheckedChanged(object sender, EventArgs e)
        {
            _sneaky.sDetectNewEnemies = cbox_detectNewEnemies.Checked;
            _sLogger.Write(string.Format("Detect new enemies: {0}", _sneaky.sDetectNewEnemies));
        }

        private void cbox_forceStealthBeforeHarvest_CheckedChanged(object sender, EventArgs e)
        {
            _sneaky.sForceStealthBeforeHarvest = cbox_forceStealthBeforeHarvest.Checked;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            ofd1.ShowDialog();
        }

        private void ofd1_FileOk(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _sneaky.LoadAvoidUnitsFromFile(ofd1.FileName);
        }
    }
}