﻿using System;
using System.Collections.Generic;
using System.Linq;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Interfaces;
using ff14bot.Managers;
using ff14bot.Objects;
using System.Threading.Tasks;
using System.Threading;
using TreeSharp;
using ff14bot.Behavior;
using ff14bot.Helpers;
using Buddy.Coroutines;
using System.IO;

namespace Sneaky
{
    public class Sneaky : IBotPlugin
    {
        internal SLogger Logger { get; private set; }
        internal List<GameObject> NearbyUnits {get;set;}
        internal List<string> AvoidUnits;
        internal int StealthTrigger = 25;
        internal bool sDetectNewEnemies = false;
        internal bool sForceStealthBeforeHarvest = false;
        private int NodeStealthTrigger = 9; // Used when settingsForceStealthBeforeHarvest is set.
        private Settings _settingsWindow;
        private Composite _coroutine;


        private Sneaky()
        {

        }

        #region Plugin Information

        public string Author
        {
            get { return " hueCannibal"; }
        }

        public string Name
        {
            get { return "Sneaky"; }
        }

        public string Description
        {
            get { return "Enter Stealth on enemy threat."; }
        }

        public Version Version
        {
            get { return new Version(0, 0, 4); }
        }


        public bool WantButton
        {
            get { return true; }
        }

        public string ButtonText
        {
            get { return "Sneaky"; }
        }

        public bool Equals(IBotPlugin other)
        {
            throw new NotImplementedException();
        }

        #endregion

        public void OnButtonPress()
        {
            if (_settingsWindow == null || _settingsWindow.IsDisposed || _settingsWindow.Disposing)
            {
                    _settingsWindow = new Settings(this);
                    _settingsWindow.Show();
            }
            else
            {
                _settingsWindow.BringToFront();
            }
        }

        public void OnInitialize()
        {
            if (Logger == null)
                Logger = new SLogger();

            NearbyUnits = new List<GameObject>();
            AvoidUnits = new List<string>();

            _coroutine = new PrioritySelector(new Decorator(c => sForceStealthBeforeHarvest && !Core.Me.InCombat && !IsStealthActive() && !Core.Me.IsCasting, new ActionRunCoroutine(r => forceStealth())),
                                              new Decorator(c => IsValidDolClass(Core.Me.CurrentJob) && !Core.Me.IsCasting && !Core.Me.IsDead, new ActionRunCoroutine(r => DetectRoutine()))
                                              );

            string DefaultAvoidListPath = @"Plugins\Sneaky\avoidList.txt";
            LoadAvoidUnitsFromFile(DefaultAvoidListPath);
        }

        internal void LoadAvoidUnitsFromFile(string path)
        {
            try
            {
                if(File.Exists(path))
                {
                    foreach(var name in File.ReadLines(path))
                    {
                        if(string.IsNullOrEmpty(name))
                            continue;

                        if(!AvoidUnits.Contains(name.Trim()))
                            AvoidUnits.Add(name.Trim());
                    }
                }
                else
                {
                    throw new IOException(string.Format("Couldn't load avoidlist from: {0}", path));
                }
            }
            catch(IOException e)
            {
                Logger.Write(LoggerType.Warning, e.Message);
            }
        }

        private async Task<bool> forceStealth()
        {
            string taskName = "[force Stealth]";
            var curPoi = Poi.Current;
            if(curPoi.Type != ff14bot.Helpers.PoiType.Gather)
            {
                Logger.Write(LoggerType.Notice, string.Format("{0} Poi type not Gather! but {1} - aborting", taskName, curPoi.Type));
                return false;
            }

            if(Core.Me.Distance(curPoi.Location) <= NodeStealthTrigger && !Core.Me.IsCasting && sForceStealthBeforeHarvest && !IsStealthActive())
            {
                if(Core.Me.IsMounted)
                    await CommonTasks.StopAndDismount();
                await CastStealth();
                Logger.Write(LoggerType.Notice, string.Format("{0} Forcing stealth.", taskName));
            }
            return false;
        }

        private void DetectNewEnemies()
        {
            if(Core.Me.CurrentTarget.Type == GameObjectType.BattleNpc || Core.Me.CurrentTarget.Type == GameObjectType.EventNpc)
            {
                var newEnemy = Core.Me.CurrentTarget.EnglishName;
                AvoidUnits.Add(newEnemy);
                _settingsWindow.ReloadAvoidList();
                _settingsWindow.DisplayNearUnits();
            }
        }

        private async Task<bool> DetectRoutine()
        {
            // Add new enemies 
            if(Core.Me.CurrentTarget != null && sDetectNewEnemies && Core.Me.InCombat)
            {
                if(!AvoidUnits.Contains(Core.Me.CurrentTarget.EnglishName))
                {
                    DetectNewEnemies();
                }
            }

            if(PlayerNearEnemy()) // Cast stealth
            {
                await CastStealth();
            }
            else // Cancel stealth
            {
                // Don't cancel stealth when the "force stealth before harvest" option is activated
                // without checking for the proper distance first.
                if(Poi.Current != null && sForceStealthBeforeHarvest)
                {
                    if(Poi.Current.Type == PoiType.Gather && Core.Me.Distance(Poi.Current.Location) <= NodeStealthTrigger)
                        return false;
                }

                await CancelStealth();
            }
            return false;
        }


        public void OnPulse()
        {
            try
            {
                if(!_coroutine.IsRunning)
                {
                    _coroutine.Start(null);
                }
            }
            catch(Exception e)
            {
                Logger.Write(LoggerType.Error, e.Message);
                throw;
            }
        }


        public void OnShutdown()
        {
        }

        public void OnDisabled()
        {
            TreeHooks.Instance.OnHooksCleared -= OnHooksCleared;
            TreeHooks.Instance.RemoveHook("TreeStart", _coroutine);
        }


        public void OnEnabled()
        {
            TreeHooks.Instance.AddHook("TreeStart", _coroutine);
            TreeHooks.Instance.OnHooksCleared += OnHooksCleared;
        }

        private void OnHooksCleared(object sender, EventArgs args)
        {
            TreeHooks.Instance.AddHook("TreeStart", _coroutine);
        }


        /// <summary>
        /// Returns True if the current job is set to either Miner, Fisher or Botanist.
        /// </summary>
        /// <param name="job"></param>
        /// <returns>True if the passed job is a gathering job, False otherwise.</returns>
        internal bool IsValidDolClass(ClassJobType job)
        {
            bool retBool;
            switch (job)
            {
                case ClassJobType.Miner:
                case ClassJobType.Fisher:
                case ClassJobType.Botanist:
                    retBool = true;
                    break;
                default:
                    retBool = false;
                    break;
            }

            return retBool;
        }

        internal List<GameObject> GetNearUnits(int range=120)
        {
            var units = GameObjectManager.GameObjects;
            var retUnits = new List<GameObject>();
            foreach(var unit in units.Where(t => t.IsMe == false && t.Type == GameObjectType.BattleNpc && t.Distance() <= range).OrderBy(r => r.Distance()))
            {
                if (!string.IsNullOrEmpty(unit.EnglishName))
                    retUnits.Add(unit);
            }

            return retUnits;
        }


        internal bool IsStealthActive()
        {
            return Core.Me.Auras.Any(aura => aura.Name == "Stealth");
        }


        internal async Task<bool> CastStealth()
        {
            if(!Actionmanager.CanCast("Stealth", null))
                return false;
            if(GatheringManager.WindowOpen)
                return false;
            if(IsStealthActive())
                return false;

            // The estimated distance until the enemy withdraws from chasing you.
            var escapeDistance = 81; 
            // Dismount to enter stealth
            var curPoi = ff14bot.Helpers.Poi.Current;
            if(curPoi.Type == ff14bot.Helpers.PoiType.Gather && Core.Me.IsMounted)
            {
                // Dismount on threat
                if(Core.Me.Distance(curPoi.Location) < escapeDistance)
                {
                    await CommonTasks.StopAndDismount();
                }
            }
            Actionmanager.DoAction("Stealth", Core.Me);
            await Coroutine.Wait(1000, () => IsStealthActive());
            return false;
        }


        internal async Task<bool> CancelStealth()
        {
            if(IsStealthActive())
            {
                Actionmanager.DoAction("Stealth", Core.Me);
                await Coroutine.Wait(2000, () => !IsStealthActive());
            }
            return false;
        }

        internal bool PlayerNearEnemy()
        {
            foreach(string avoidUnitName in AvoidUnits.Where(u => u != null))
            {
                var currentNearbyUnits = GetNearUnits();
                if(currentNearbyUnits.Any())
                {
                    if(currentNearbyUnits.Where(u => u.EnglishName == avoidUnitName && !u.IsMe
                        && (u.Type == GameObjectType.BattleNpc || u.Type == GameObjectType.EventNpc)
                        && Core.Me.Distance(u) <= StealthTrigger).Any())
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }
}